#ifndef ISHEETT_H
#define ISHEETT_H

#include "setup.h"
#include "enum.h"

namespace libxl {

    template<class TCHAR> struct IFormatT;

    template<class TCHAR>
    struct ISheetT
    {
        virtual       CellType XLAPIENTRY cellType(unsigned short row, unsigned short col) const = 0;

        virtual   const TCHAR* XLAPIENTRY readStr(unsigned short row, unsigned short col, IFormatT<TCHAR>** format = 0) = 0;
        virtual           bool XLAPIENTRY writeStr(unsigned short row, unsigned short col, const TCHAR* value, IFormatT<TCHAR>* format = 0) = 0;

        virtual         double XLAPIENTRY readNum(unsigned short row, unsigned short col, IFormatT<TCHAR>** format = 0) const = 0;
        virtual           bool XLAPIENTRY writeNum(unsigned short row, unsigned short col, double value, IFormatT<TCHAR>* format = 0) = 0;

        virtual           bool XLAPIENTRY readBool(unsigned short row, unsigned short col, IFormatT<TCHAR>** format = 0) const = 0;
        virtual           bool XLAPIENTRY writeBool(unsigned short row, unsigned short col, bool value, IFormatT<TCHAR>* format = 0) = 0;

        virtual           bool XLAPIENTRY readBlank(unsigned short row, unsigned short col, IFormatT<TCHAR>** format) const = 0;
        virtual           bool XLAPIENTRY writeBlank(unsigned short row, unsigned short col, IFormatT<TCHAR>* format) = 0;

        virtual   const TCHAR* XLAPIENTRY readComment(unsigned short row, unsigned short col) const = 0;
        virtual           void XLAPIENTRY writeComment(unsigned short row, unsigned short col, const TCHAR* value, const TCHAR* author = 0, int width = 129, int height = 75) = 0;

        virtual           bool XLAPIENTRY isDate(unsigned short row, unsigned short col) const = 0;
        virtual      ErrorType XLAPIENTRY readError(unsigned short row, unsigned short col) const = 0;

        virtual         double XLAPIENTRY colWidth(unsigned short col) const = 0;
        virtual         double XLAPIENTRY rowHeight(unsigned short row) const = 0;

        virtual           bool XLAPIENTRY setCol(unsigned short colFirst, unsigned short colLast, double width, IFormatT<TCHAR>* format = 0, bool hidden = false) = 0;
        virtual           bool XLAPIENTRY setRow(unsigned short row, double height, IFormatT<TCHAR>* format = 0, bool hidden = false) = 0;

        virtual           bool XLAPIENTRY getMerge(unsigned short row, unsigned short col, unsigned short* rowFirst, unsigned short* rowLast, unsigned short* colFirst, unsigned short* colLast) = 0;
        virtual           bool XLAPIENTRY setMerge(unsigned short rowFirst, unsigned short rowLast, unsigned short colFirst, unsigned short colLast) = 0;

        virtual           void XLAPIENTRY setPicture(unsigned short row, unsigned short col, int pictureId, double scale = 1.0) = 0;
        virtual           void XLAPIENTRY setPicture2(unsigned short row, unsigned short col, int pictureId, int width, int height) = 0;
       
        virtual           bool XLAPIENTRY setHorPageBreak(unsigned short row, bool pageBreak = true) = 0;
        virtual           bool XLAPIENTRY setVerPageBreak(unsigned short col, bool pageBreak = true) = 0;

        virtual           void XLAPIENTRY split(unsigned short row, unsigned short col) = 0;
      
        virtual           bool XLAPIENTRY groupRows(unsigned short rowFirst, unsigned rowLast, bool collapsed = true) = 0;
        virtual           bool XLAPIENTRY groupCols(unsigned short colFirst, unsigned colLast, bool collapsed = true) = 0;

        virtual           void XLAPIENTRY clear(unsigned short rowFirst = 0, unsigned short rowLast = 65535, unsigned short colFirst = 0, unsigned short colLast = 255) = 0;

        virtual unsigned short XLAPIENTRY firstRow() const = 0;
        virtual unsigned short XLAPIENTRY lastRow() const = 0;
        virtual unsigned short XLAPIENTRY firstCol() const = 0;
        virtual unsigned short XLAPIENTRY lastCol() const = 0;
                       
        virtual           bool XLAPIENTRY displayGridlines() const = 0;
        virtual           void XLAPIENTRY setDisplayGridlines(bool show = true) = 0;

        virtual           bool XLAPIENTRY printGridlines() const = 0;
        virtual           void XLAPIENTRY setPrintGridLines(bool print = true) = 0;
   
        virtual       unsigned XLAPIENTRY zoom() const = 0;
        virtual           void XLAPIENTRY setZoom(unsigned zoom) = 0;

        virtual       unsigned XLAPIENTRY printZoom() const = 0;
        virtual           void XLAPIENTRY setPrintZoom(unsigned zoom) = 0;

        virtual           bool XLAPIENTRY landscape() const = 0;
        virtual           void XLAPIENTRY setLandscape(bool landscape = true) = 0;

        virtual          Paper XLAPIENTRY paper() const = 0;
        virtual           void XLAPIENTRY setPaper(Paper paper = PAPER_DEFAULT) = 0;
     
        virtual   const TCHAR* XLAPIENTRY header() const = 0;
        virtual           bool XLAPIENTRY setHeader(const TCHAR* header, double margin = 0.5) = 0;
        virtual         double XLAPIENTRY headerMargin() const = 0;
    
        virtual   const TCHAR* XLAPIENTRY footer() const = 0;    
        virtual           bool XLAPIENTRY setFooter(const TCHAR* footer, double margin = 0.5) = 0;
        virtual         double XLAPIENTRY footerMargin() const = 0;

        virtual           bool XLAPIENTRY hCenter() const = 0;
        virtual           void XLAPIENTRY setHCenter(bool hCenter = true) = 0;

        virtual           bool XLAPIENTRY vCenter() const = 0;
        virtual           void XLAPIENTRY setVCenter(bool vCenter = true) = 0;

        virtual         double XLAPIENTRY marginLeft() const = 0;
        virtual           void XLAPIENTRY setMarginLeft(double margin) = 0;

        virtual         double XLAPIENTRY marginRight() const = 0;
        virtual           void XLAPIENTRY setMarginRight(double margin) = 0;

        virtual         double XLAPIENTRY marginTop() const = 0;
        virtual           void XLAPIENTRY setMarginTop(double margin) = 0;
 
        virtual         double XLAPIENTRY marginBottom() const = 0;
        virtual           void XLAPIENTRY setMarginBottom(double margin) = 0;

        virtual           bool XLAPIENTRY printRowCol() const = 0;
        virtual           void XLAPIENTRY setPrintRowCol(bool print = true) = 0;
    
        virtual   const TCHAR* XLAPIENTRY name() const = 0;       
    };

}

#endif
